/*******************************************************************************
*    DOGM-Library 1.0.                                                         *
*    Copyright (C) 2010-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#ifndef DOGM_H_
#define DOGM_H_

#define DOGMVERSION                 "1.0-20101023"

#include <avr/io.h>

#define LCD_BACKLIGHT_PORT          PORTC
#define LCD_BACKLIGHT_DDR           DDRC
#define LCD_BACKLIGHT               1

#define LCD_E_PORT                  PORTC
#define LCD_E_DDR                   DDRC
#define LCD_ENABLE                  3

#define LCD_RW_PORT                 PORTC
#define LCD_RW_DDR                  DDRC
#define LCD_RW                      2

#define LCD_RS_PORT                 PORTC
#define LCD_RS_DDR                  DDRC
#define LCD_REGSELECT               0

#define LCD_DATA_PORT               PORTC
#define LCD_DATA_DDR                DDRC
#define LCD_DATA_PIN                PINC
#define LCD_DATA7                   7
#define LCD_DATA6                   6
#define LCD_DATA5                   5
#define LCD_DATA4                   4

#define POWER5V                     0
#define POWER3V3                    1
#define LCD_LINES                   3
#define LCD_COLS                    16

/**
 * Reads a byte from LCD-controller
 *
 * @param uint8_t rs        0: read busy flag / address counter
 *                          1: read data
 * @return uint8_t          the data
 */
uint8_t lcdReadByte(uint8_t rs);

/**
 * Sends a byte to the LCD-controller
 *
 * @param char data         the byte
 */
void lcdSendByte(char data);

/**
 * Sends an instruction to the LCD-controller
 *
 * @param char instruction  the instruction
 */
void lcdSendInstruction(char instruction);

/**
 * Writes a char on the LCD-display
 *
 * @param char c            the character
 */
void lcdPutc(char c);

/**
 * Writes a string on the LCD-display.
 * \n will accepted as Carriage Return
 *
 * @param char *str         the string
 */
void lcdPuts(char *str);

/**
 * Deletes the content of the LCD-display
 */
void lcdClear(void);

/**
 * Sets the contrast of the LCD-display
 *
 * @param uint8_t contrast  the contrast (4 bits!!)
 */
void lcdContrast(uint8_t contrast);

/**
 * Moves the cursor to the position x/y
 *
 * @param uint8_t x         the horizontal position
 * @param uint8_t y         the vertical position
 */
void lcdGotoXY(uint8_t x, uint8_t y);

/**
 * Reads the current cursor's position
 *
 * @param uint8_t *x        the horizontal position (as reference)
 * @param uint8_t *y        the vertical position (as reference)
 */
void lcdGetXY(uint8_t *x, uint8_t *y);

/**
 * Initialises the LCD-controller
 *
 * @param uint8_t power     POWER5V or POWER3V3
 * @param uint8_t contrast  the initial contrast
 * @param uint8_t cursor    1 if the cursor should be displayed, 0 otherwise
 * @param uint8_t blink     1 if the cursor should blink, 0 otherwise
 */
void lcdInit(uint8_t power, uint8_t contrast, uint8_t cursor, uint8_t blink);

/**
 * Turns the LCD-backlight on
 */
#define lcdBacklightOn()            LCD_BACKLIGHT_PORT  |=  _BV(LCD_BACKLIGHT);

/**
 * Turns the LCD-backlight off
 */
#define lcdBacklightOff()           LCD_BACKLIGHT_PORT  &= ~_BV(LCD_BACKLIGHT);

#endif /* DOGM_H_ */
